# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A primitive representing a text label
class TextPrimitive(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = TextPrimitive()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsTextPrimitive(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # TextPrimitive
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Position of the center of the text box and orientation of the text. Identity orientation means the text is oriented in the xy-plane and flows from -x to +x.
    # TextPrimitive
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Whether the text should respect `pose.orientation` (false) or always face the camera (true)
    # TextPrimitive
    def Billboard(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Font size (height of one line of text)
    # TextPrimitive
    def FontSize(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Indicates whether `font_size` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)
    # TextPrimitive
    def ScaleInvariant(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Color of the text
    # TextPrimitive
    def Color(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Text
    # TextPrimitive
    def Text(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def TextPrimitiveStart(builder): builder.StartObject(6)
def Start(builder):
    return TextPrimitiveStart(builder)
def TextPrimitiveAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return TextPrimitiveAddPose(builder, pose)
def TextPrimitiveAddBillboard(builder, billboard): builder.PrependBoolSlot(1, billboard, 0)
def AddBillboard(builder, billboard):
    return TextPrimitiveAddBillboard(builder, billboard)
def TextPrimitiveAddFontSize(builder, fontSize): builder.PrependFloat64Slot(2, fontSize, 0.0)
def AddFontSize(builder, fontSize):
    return TextPrimitiveAddFontSize(builder, fontSize)
def TextPrimitiveAddScaleInvariant(builder, scaleInvariant): builder.PrependBoolSlot(3, scaleInvariant, 0)
def AddScaleInvariant(builder, scaleInvariant):
    return TextPrimitiveAddScaleInvariant(builder, scaleInvariant)
def TextPrimitiveAddColor(builder, color): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(color), 0)
def AddColor(builder, color):
    return TextPrimitiveAddColor(builder, color)
def TextPrimitiveAddText(builder, text): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(text), 0)
def AddText(builder, text):
    return TextPrimitiveAddText(builder, text)
def TextPrimitiveEnd(builder): return builder.EndObject()
def End(builder):
    return TextPrimitiveEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A primitive representing an arrow
class ArrowPrimitive(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ArrowPrimitive()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsArrowPrimitive(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # ArrowPrimitive
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Position of the arrow's tail and orientation of the arrow. Identity orientation means the arrow points in the +x direction.
    # ArrowPrimitive
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Length of the arrow shaft
    # ArrowPrimitive
    def ShaftLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Diameter of the arrow shaft
    # ArrowPrimitive
    def ShaftDiameter(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Length of the arrow head
    # ArrowPrimitive
    def HeadLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Diameter of the arrow head
    # ArrowPrimitive
    def HeadDiameter(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Color of the arrow
    # ArrowPrimitive
    def Color(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def ArrowPrimitiveStart(builder): builder.StartObject(6)
def Start(builder):
    return ArrowPrimitiveStart(builder)
def ArrowPrimitiveAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return ArrowPrimitiveAddPose(builder, pose)
def ArrowPrimitiveAddShaftLength(builder, shaftLength): builder.PrependFloat64Slot(1, shaftLength, 0.0)
def AddShaftLength(builder, shaftLength):
    return ArrowPrimitiveAddShaftLength(builder, shaftLength)
def ArrowPrimitiveAddShaftDiameter(builder, shaftDiameter): builder.PrependFloat64Slot(2, shaftDiameter, 0.0)
def AddShaftDiameter(builder, shaftDiameter):
    return ArrowPrimitiveAddShaftDiameter(builder, shaftDiameter)
def ArrowPrimitiveAddHeadLength(builder, headLength): builder.PrependFloat64Slot(3, headLength, 0.0)
def AddHeadLength(builder, headLength):
    return ArrowPrimitiveAddHeadLength(builder, headLength)
def ArrowPrimitiveAddHeadDiameter(builder, headDiameter): builder.PrependFloat64Slot(4, headDiameter, 0.0)
def AddHeadDiameter(builder, headDiameter):
    return ArrowPrimitiveAddHeadDiameter(builder, headDiameter)
def ArrowPrimitiveAddColor(builder, color): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(color), 0)
def AddColor(builder, color):
    return ArrowPrimitiveAddColor(builder, color)
def ArrowPrimitiveEnd(builder): return builder.EndObject()
def End(builder):
    return ArrowPrimitiveEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# Array of annotations for a 2D image
class ImageAnnotations(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ImageAnnotations()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsImageAnnotations(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # ImageAnnotations
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Circle annotations
    # ImageAnnotations
    def Circles(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .CircleAnnotation import CircleAnnotation
            obj = CircleAnnotation()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # ImageAnnotations
    def CirclesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # ImageAnnotations
    def CirclesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

    # Points annotations
    # ImageAnnotations
    def Points(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .PointsAnnotation import PointsAnnotation
            obj = PointsAnnotation()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # ImageAnnotations
    def PointsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # ImageAnnotations
    def PointsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        return o == 0

    # Text annotations
    # ImageAnnotations
    def Texts(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .TextAnnotation import TextAnnotation
            obj = TextAnnotation()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # ImageAnnotations
    def TextsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # ImageAnnotations
    def TextsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

def ImageAnnotationsStart(builder): builder.StartObject(3)
def Start(builder):
    return ImageAnnotationsStart(builder)
def ImageAnnotationsAddCircles(builder, circles): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(circles), 0)
def AddCircles(builder, circles):
    return ImageAnnotationsAddCircles(builder, circles)
def ImageAnnotationsStartCirclesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartCirclesVector(builder, numElems):
    return ImageAnnotationsStartCirclesVector(builder, numElems)
def ImageAnnotationsAddPoints(builder, points): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(points), 0)
def AddPoints(builder, points):
    return ImageAnnotationsAddPoints(builder, points)
def ImageAnnotationsStartPointsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartPointsVector(builder, numElems):
    return ImageAnnotationsStartPointsVector(builder, numElems)
def ImageAnnotationsAddTexts(builder, texts): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(texts), 0)
def AddTexts(builder, texts):
    return ImageAnnotationsAddTexts(builder, texts)
def ImageAnnotationsStartTextsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartTextsVector(builder, numElems):
    return ImageAnnotationsStartTextsVector(builder, numElems)
def ImageAnnotationsEnd(builder): return builder.EndObject()
def End(builder):
    return ImageAnnotationsEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A visual element in a 3D scene. An entity may be composed of multiple primitives which all share the same frame of reference.
class SceneEntity(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = SceneEntity()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsSceneEntity(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # SceneEntity
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of the entity
    # SceneEntity
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference
    # SceneEntity
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Identifier for the entity. A entity will replace any prior entity on the same topic with the same `id`.
    # SceneEntity
    def Id(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Length of time (relative to `timestamp`) after which the entity should be automatically removed. Zero value indicates the entity should remain visible until it is replaced or deleted.
    # SceneEntity
    def Lifetime(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = o + self._tab.Pos
            from .Duration import Duration
            obj = Duration()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Whether the entity should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    # SceneEntity
    def FrameLocked(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Additional user-provided metadata associated with the entity. Keys must be unique.
    # SceneEntity
    def Metadata(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .KeyValuePair import KeyValuePair
            obj = KeyValuePair()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def MetadataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def MetadataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        return o == 0

    # Arrow primitives
    # SceneEntity
    def Arrows(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .ArrowPrimitive import ArrowPrimitive
            obj = ArrowPrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def ArrowsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def ArrowsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        return o == 0

    # Cube primitives
    # SceneEntity
    def Cubes(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .CubePrimitive import CubePrimitive
            obj = CubePrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def CubesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def CubesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        return o == 0

    # Sphere primitives
    # SceneEntity
    def Spheres(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .SpherePrimitive import SpherePrimitive
            obj = SpherePrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def SpheresLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def SpheresIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        return o == 0

    # Cylinder primitives
    # SceneEntity
    def Cylinders(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .CylinderPrimitive import CylinderPrimitive
            obj = CylinderPrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def CylindersLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def CylindersIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        return o == 0

    # Line primitives
    # SceneEntity
    def Lines(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .LinePrimitive import LinePrimitive
            obj = LinePrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def LinesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def LinesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        return o == 0

    # Triangle list primitives
    # SceneEntity
    def Triangles(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .TriangleListPrimitive import TriangleListPrimitive
            obj = TriangleListPrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def TrianglesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def TrianglesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        return o == 0

    # Text primitives
    # SceneEntity
    def Texts(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(28))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .TextPrimitive import TextPrimitive
            obj = TextPrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def TextsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(28))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def TextsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(28))
        return o == 0

    # Model primitives
    # SceneEntity
    def Models(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(30))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .ModelPrimitive import ModelPrimitive
            obj = ModelPrimitive()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneEntity
    def ModelsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(30))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneEntity
    def ModelsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(30))
        return o == 0

def SceneEntityStart(builder): builder.StartObject(14)
def Start(builder):
    return SceneEntityStart(builder)
def SceneEntityAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return SceneEntityAddTimestamp(builder, timestamp)
def SceneEntityAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return SceneEntityAddFrameId(builder, frameId)
def SceneEntityAddId(builder, id): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(id), 0)
def AddId(builder, id):
    return SceneEntityAddId(builder, id)
def SceneEntityAddLifetime(builder, lifetime): builder.PrependStructSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(lifetime), 0)
def AddLifetime(builder, lifetime):
    return SceneEntityAddLifetime(builder, lifetime)
def SceneEntityAddFrameLocked(builder, frameLocked): builder.PrependBoolSlot(4, frameLocked, 0)
def AddFrameLocked(builder, frameLocked):
    return SceneEntityAddFrameLocked(builder, frameLocked)
def SceneEntityAddMetadata(builder, metadata): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(metadata), 0)
def AddMetadata(builder, metadata):
    return SceneEntityAddMetadata(builder, metadata)
def SceneEntityStartMetadataVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartMetadataVector(builder, numElems):
    return SceneEntityStartMetadataVector(builder, numElems)
def SceneEntityAddArrows(builder, arrows): builder.PrependUOffsetTRelativeSlot(6, flatbuffers.number_types.UOffsetTFlags.py_type(arrows), 0)
def AddArrows(builder, arrows):
    return SceneEntityAddArrows(builder, arrows)
def SceneEntityStartArrowsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartArrowsVector(builder, numElems):
    return SceneEntityStartArrowsVector(builder, numElems)
def SceneEntityAddCubes(builder, cubes): builder.PrependUOffsetTRelativeSlot(7, flatbuffers.number_types.UOffsetTFlags.py_type(cubes), 0)
def AddCubes(builder, cubes):
    return SceneEntityAddCubes(builder, cubes)
def SceneEntityStartCubesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartCubesVector(builder, numElems):
    return SceneEntityStartCubesVector(builder, numElems)
def SceneEntityAddSpheres(builder, spheres): builder.PrependUOffsetTRelativeSlot(8, flatbuffers.number_types.UOffsetTFlags.py_type(spheres), 0)
def AddSpheres(builder, spheres):
    return SceneEntityAddSpheres(builder, spheres)
def SceneEntityStartSpheresVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartSpheresVector(builder, numElems):
    return SceneEntityStartSpheresVector(builder, numElems)
def SceneEntityAddCylinders(builder, cylinders): builder.PrependUOffsetTRelativeSlot(9, flatbuffers.number_types.UOffsetTFlags.py_type(cylinders), 0)
def AddCylinders(builder, cylinders):
    return SceneEntityAddCylinders(builder, cylinders)
def SceneEntityStartCylindersVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartCylindersVector(builder, numElems):
    return SceneEntityStartCylindersVector(builder, numElems)
def SceneEntityAddLines(builder, lines): builder.PrependUOffsetTRelativeSlot(10, flatbuffers.number_types.UOffsetTFlags.py_type(lines), 0)
def AddLines(builder, lines):
    return SceneEntityAddLines(builder, lines)
def SceneEntityStartLinesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartLinesVector(builder, numElems):
    return SceneEntityStartLinesVector(builder, numElems)
def SceneEntityAddTriangles(builder, triangles): builder.PrependUOffsetTRelativeSlot(11, flatbuffers.number_types.UOffsetTFlags.py_type(triangles), 0)
def AddTriangles(builder, triangles):
    return SceneEntityAddTriangles(builder, triangles)
def SceneEntityStartTrianglesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartTrianglesVector(builder, numElems):
    return SceneEntityStartTrianglesVector(builder, numElems)
def SceneEntityAddTexts(builder, texts): builder.PrependUOffsetTRelativeSlot(12, flatbuffers.number_types.UOffsetTFlags.py_type(texts), 0)
def AddTexts(builder, texts):
    return SceneEntityAddTexts(builder, texts)
def SceneEntityStartTextsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartTextsVector(builder, numElems):
    return SceneEntityStartTextsVector(builder, numElems)
def SceneEntityAddModels(builder, models): builder.PrependUOffsetTRelativeSlot(13, flatbuffers.number_types.UOffsetTFlags.py_type(models), 0)
def AddModels(builder, models):
    return SceneEntityAddModels(builder, models)
def SceneEntityStartModelsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartModelsVector(builder, numElems):
    return SceneEntityStartModelsVector(builder, numElems)
def SceneEntityEnd(builder): return builder.EndObject()
def End(builder):
    return SceneEntityEnd(builder)


def test_schemas():
    import flatbuffers
    import foxglove_schemas_flatbuffer.RawImage as RawImage

    builder = flatbuffers.Builder(1024)
    png = builder.CreateString("png")
    RawImage.Start(builder)
    RawImage.AddEncoding(builder, png)
    img = RawImage.End(builder)
    builder.Finish(img)
    buf = builder.Output()
    assert buf is not None
    assert len(buf) > 0


def test_get_schema():
    from foxglove_schemas_flatbuffer import get_schema

    schema = get_schema("RawImage")
    assert schema is not None
    assert len(schema) > 0



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A color in RGBA format
class Color(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Color()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsColor(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Color
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Red value between 0 and 1
    # Color
    def R(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 1.0

    # Green value between 0 and 1
    # Color
    def G(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 1.0

    # Blue value between 0 and 1
    # Color
    def B(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 1.0

    # Alpha value between 0 and 1
    # Color
    def A(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 1.0

def ColorStart(builder): builder.StartObject(4)
def Start(builder):
    return ColorStart(builder)
def ColorAddR(builder, r): builder.PrependFloat64Slot(0, r, 1.0)
def AddR(builder, r):
    return ColorAddR(builder, r)
def ColorAddG(builder, g): builder.PrependFloat64Slot(1, g, 1.0)
def AddG(builder, g):
    return ColorAddG(builder, g)
def ColorAddB(builder, b): builder.PrependFloat64Slot(2, b, 1.0)
def AddB(builder, b):
    return ColorAddB(builder, b)
def ColorAddA(builder, a): builder.PrependFloat64Slot(3, a, 1.0)
def AddA(builder, a):
    return ColorAddA(builder, a)
def ColorEnd(builder): return builder.EndObject()
def End(builder):
    return ColorEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A single scan from a planar laser range-finder
class LaserScan(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = LaserScan()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsLaserScan(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # LaserScan
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of scan
    # LaserScan
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference
    # LaserScan
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Origin of scan relative to frame of reference; points are positioned in the x-y plane relative to this origin; angles are interpreted as counterclockwise rotations around the z axis with 0 rad being in the +x direction
    # LaserScan
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Bearing of first point, in radians
    # LaserScan
    def StartAngle(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Bearing of last point, in radians
    # LaserScan
    def EndAngle(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Distance of detections from origin; assumed to be at equally-spaced angles between `start_angle` and `end_angle`
    # LaserScan
    def Ranges(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # LaserScan
    def RangesAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # LaserScan
    def RangesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # LaserScan
    def RangesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        return o == 0

    # Intensity of detections
    # LaserScan
    def Intensities(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # LaserScan
    def IntensitiesAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # LaserScan
    def IntensitiesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # LaserScan
    def IntensitiesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        return o == 0

def LaserScanStart(builder): builder.StartObject(7)
def Start(builder):
    return LaserScanStart(builder)
def LaserScanAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return LaserScanAddTimestamp(builder, timestamp)
def LaserScanAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return LaserScanAddFrameId(builder, frameId)
def LaserScanAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return LaserScanAddPose(builder, pose)
def LaserScanAddStartAngle(builder, startAngle): builder.PrependFloat64Slot(3, startAngle, 0.0)
def AddStartAngle(builder, startAngle):
    return LaserScanAddStartAngle(builder, startAngle)
def LaserScanAddEndAngle(builder, endAngle): builder.PrependFloat64Slot(4, endAngle, 0.0)
def AddEndAngle(builder, endAngle):
    return LaserScanAddEndAngle(builder, endAngle)
def LaserScanAddRanges(builder, ranges): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(ranges), 0)
def AddRanges(builder, ranges):
    return LaserScanAddRanges(builder, ranges)
def LaserScanStartRangesVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def StartRangesVector(builder, numElems):
    return LaserScanStartRangesVector(builder, numElems)
def LaserScanAddIntensities(builder, intensities): builder.PrependUOffsetTRelativeSlot(6, flatbuffers.number_types.UOffsetTFlags.py_type(intensities), 0)
def AddIntensities(builder, intensities):
    return LaserScanAddIntensities(builder, intensities)
def LaserScanStartIntensitiesVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def StartIntensitiesVector(builder, numElems):
    return LaserScanStartIntensitiesVector(builder, numElems)
def LaserScanEnd(builder): return builder.EndObject()
def End(builder):
    return LaserScanEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

# An enumeration indicating which entities should match a SceneEntityDeletion command
class SceneEntityDeletionType(object):
    # Delete the existing entity on the same topic that has the provided `id`
    MATCHING_ID = 0
    # Delete all existing entities on the same topic
    ALL = 1



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# Camera calibration parameters
class CameraCalibration(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = CameraCalibration()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsCameraCalibration(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # CameraCalibration
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of calibration data
    # CameraCalibration
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference for the camera. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image.
    # CameraCalibration
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Image width
    # CameraCalibration
    def Width(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Image height
    # CameraCalibration
    def Height(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Name of distortion model
    # 
    # Supported parameters: `plumb_bob` (k1, k2, p1, p2, k3) and `rational_polynomial` (k1, k2, p1, p2, k3, k4, k5, k6). Distortion models are based on [OpenCV's](https://docs.opencv.org/2.4/modules/calib3d/doc/camera_calibration_and_3d_reconstruction.html) [pinhole camera model](https://en.wikipedia.org/wiki/Distortion_%28optics%29#Software_correction). This is the same [implementation used by ROS](http://docs.ros.org/en/diamondback/api/image_geometry/html/c++/pinhole__camera__model_8cpp_source.html)
    # CameraCalibration
    def DistortionModel(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Distortion parameters
    # CameraCalibration
    def D(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # CameraCalibration
    def DAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # CameraCalibration
    def DLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CameraCalibration
    def DIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        return o == 0

    # Intrinsic camera matrix (3x3 row-major matrix)
    # 
    # A 3x3 row-major matrix for the raw (distorted) image.
    # 
    # Projects 3D points in the camera coordinate frame to 2D pixel coordinates using the focal lengths (fx, fy) and principal point (cx, cy).
    # 
    # ```
    #     [fx  0 cx]
    # K = [ 0 fy cy]
    #     [ 0  0  1]
    # ```
    # length 9
    # CameraCalibration
    def K(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # CameraCalibration
    def KAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # CameraCalibration
    def KLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CameraCalibration
    def KIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        return o == 0

    # Rectification matrix (stereo cameras only, 3x3 row-major matrix)
    # 
    # A rotation matrix aligning the camera coordinate system to the ideal stereo image plane so that epipolar lines in both stereo images are parallel.
    # length 9
    # CameraCalibration
    def R(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # CameraCalibration
    def RAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # CameraCalibration
    def RLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CameraCalibration
    def RIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        return o == 0

    # Projection/camera matrix (3x4 row-major matrix)
    # 
    # ```
    #     [fx'  0  cx' Tx]
    # P = [ 0  fy' cy' Ty]
    #     [ 0   0   1   0]
    # ```
    # 
    # By convention, this matrix specifies the intrinsic (camera) matrix of the processed (rectified) image. That is, the left 3x3 portion is the normal camera intrinsic matrix for the rectified image.
    # 
    # It projects 3D points in the camera coordinate frame to 2D pixel coordinates using the focal lengths (fx', fy') and principal point (cx', cy') - these may differ from the values in K.
    # 
    # For monocular cameras, Tx = Ty = 0. Normally, monocular cameras will also have R = the identity and P[1:3,1:3] = K.
    # 
    # For a stereo pair, the fourth column [Tx Ty 0]' is related to the position of the optical center of the second camera in the first camera's frame. We assume Tz = 0 so both cameras are in the same stereo image plane. The first camera always has Tx = Ty = 0. For the right (second) camera of a horizontal stereo pair, Ty = 0 and Tx = -fx' * B, where B is the baseline between the cameras.
    # 
    # Given a 3D point [X Y Z]', the projection (x, y) of the point onto the rectified image is given by:
    # 
    # ```
    # [u v w]' = P * [X Y Z 1]'
    #        x = u / w
    #        y = v / w
    # ```
    # 
    # This holds for both images of a stereo pair.
    # length 12
    # CameraCalibration
    def P(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # CameraCalibration
    def PAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # CameraCalibration
    def PLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CameraCalibration
    def PIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        return o == 0

def CameraCalibrationStart(builder): builder.StartObject(9)
def Start(builder):
    return CameraCalibrationStart(builder)
def CameraCalibrationAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return CameraCalibrationAddTimestamp(builder, timestamp)
def CameraCalibrationAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return CameraCalibrationAddFrameId(builder, frameId)
def CameraCalibrationAddWidth(builder, width): builder.PrependUint32Slot(2, width, 0)
def AddWidth(builder, width):
    return CameraCalibrationAddWidth(builder, width)
def CameraCalibrationAddHeight(builder, height): builder.PrependUint32Slot(3, height, 0)
def AddHeight(builder, height):
    return CameraCalibrationAddHeight(builder, height)
def CameraCalibrationAddDistortionModel(builder, distortionModel): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(distortionModel), 0)
def AddDistortionModel(builder, distortionModel):
    return CameraCalibrationAddDistortionModel(builder, distortionModel)
def CameraCalibrationAddD(builder, d): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(d), 0)
def AddD(builder, d):
    return CameraCalibrationAddD(builder, d)
def CameraCalibrationStartDVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def StartDVector(builder, numElems):
    return CameraCalibrationStartDVector(builder, numElems)
def CameraCalibrationAddK(builder, k): builder.PrependUOffsetTRelativeSlot(6, flatbuffers.number_types.UOffsetTFlags.py_type(k), 0)
def AddK(builder, k):
    return CameraCalibrationAddK(builder, k)
def CameraCalibrationStartKVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def StartKVector(builder, numElems):
    return CameraCalibrationStartKVector(builder, numElems)
def CameraCalibrationAddR(builder, r): builder.PrependUOffsetTRelativeSlot(7, flatbuffers.number_types.UOffsetTFlags.py_type(r), 0)
def AddR(builder, r):
    return CameraCalibrationAddR(builder, r)
def CameraCalibrationStartRVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def StartRVector(builder, numElems):
    return CameraCalibrationStartRVector(builder, numElems)
def CameraCalibrationAddP(builder, p): builder.PrependUOffsetTRelativeSlot(8, flatbuffers.number_types.UOffsetTFlags.py_type(p), 0)
def AddP(builder, p):
    return CameraCalibrationAddP(builder, p)
def CameraCalibrationStartPVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def StartPVector(builder, numElems):
    return CameraCalibrationStartPVector(builder, numElems)
def CameraCalibrationEnd(builder): return builder.EndObject()
def End(builder):
    return CameraCalibrationEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A circle annotation on a 2D image
class CircleAnnotation(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = CircleAnnotation()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsCircleAnnotation(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # CircleAnnotation
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of circle
    # CircleAnnotation
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Center of the circle in 2D image coordinates (pixels).
    # The coordinate uses the top-left corner of the top-left pixel of the image as the origin.
    # CircleAnnotation
    def Position(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Point2 import Point2
            obj = Point2()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Circle diameter in pixels
    # CircleAnnotation
    def Diameter(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Line thickness in pixels
    # CircleAnnotation
    def Thickness(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Fill color
    # CircleAnnotation
    def FillColor(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Outline color
    # CircleAnnotation
    def OutlineColor(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def CircleAnnotationStart(builder): builder.StartObject(6)
def Start(builder):
    return CircleAnnotationStart(builder)
def CircleAnnotationAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return CircleAnnotationAddTimestamp(builder, timestamp)
def CircleAnnotationAddPosition(builder, position): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(position), 0)
def AddPosition(builder, position):
    return CircleAnnotationAddPosition(builder, position)
def CircleAnnotationAddDiameter(builder, diameter): builder.PrependFloat64Slot(2, diameter, 0.0)
def AddDiameter(builder, diameter):
    return CircleAnnotationAddDiameter(builder, diameter)
def CircleAnnotationAddThickness(builder, thickness): builder.PrependFloat64Slot(3, thickness, 0.0)
def AddThickness(builder, thickness):
    return CircleAnnotationAddThickness(builder, thickness)
def CircleAnnotationAddFillColor(builder, fillColor): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(fillColor), 0)
def AddFillColor(builder, fillColor):
    return CircleAnnotationAddFillColor(builder, fillColor)
def CircleAnnotationAddOutlineColor(builder, outlineColor): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(outlineColor), 0)
def AddOutlineColor(builder, outlineColor):
    return CircleAnnotationAddOutlineColor(builder, outlineColor)
def CircleAnnotationEnd(builder): return builder.EndObject()
def End(builder):
    return CircleAnnotationEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A collection of N-dimensional points, which may contain additional fields with information like normals, intensity, etc.
class PointCloud(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = PointCloud()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsPointCloud(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # PointCloud
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of point cloud
    # PointCloud
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference
    # PointCloud
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # The origin of the point cloud relative to the frame of reference
    # PointCloud
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Number of bytes between points in the `data`
    # PointCloud
    def PointStride(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Fields in `data`. At least 2 coordinate fields from `x`, `y`, and `z` are required for each point's position; `red`, `green`, `blue`, and `alpha` are optional for customizing each point's color.
    # PointCloud
    def Fields(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .PackedElementField import PackedElementField
            obj = PackedElementField()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # PointCloud
    def FieldsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # PointCloud
    def FieldsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        return o == 0

    # Point data, interpreted using `fields`
    # PointCloud
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # PointCloud
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # PointCloud
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # PointCloud
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        return o == 0

def PointCloudStart(builder): builder.StartObject(6)
def Start(builder):
    return PointCloudStart(builder)
def PointCloudAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return PointCloudAddTimestamp(builder, timestamp)
def PointCloudAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return PointCloudAddFrameId(builder, frameId)
def PointCloudAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return PointCloudAddPose(builder, pose)
def PointCloudAddPointStride(builder, pointStride): builder.PrependUint32Slot(3, pointStride, 0)
def AddPointStride(builder, pointStride):
    return PointCloudAddPointStride(builder, pointStride)
def PointCloudAddFields(builder, fields): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(fields), 0)
def AddFields(builder, fields):
    return PointCloudAddFields(builder, fields)
def PointCloudStartFieldsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartFieldsVector(builder, numElems):
    return PointCloudStartFieldsVector(builder, numElems)
def PointCloudAddData(builder, data): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def AddData(builder, data):
    return PointCloudAddData(builder, data)
def PointCloudStartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def StartDataVector(builder, numElems):
    return PointCloudStartDataVector(builder, numElems)
def PointCloudEnd(builder): return builder.EndObject()
def End(builder):
    return PointCloudEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A single frame of a compressed video bitstream
class CompressedVideo(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = CompressedVideo()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsCompressedVideo(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # CompressedVideo
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of video frame
    # CompressedVideo
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference for the video.
    # 
    # The origin of the frame is the optical center of the camera. +x points to the right in the video, +y points down, and +z points into the plane of the video.
    # CompressedVideo
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Compressed video frame data.
    # 
    # For packet-based video codecs this data must begin and end on packet boundaries (no partial packets), and must contain enough video packets to decode exactly one image (either a keyframe or delta frame). Note: Foxglove does not support video streams that include B frames because they require lookahead.
    # 
    # Specifically, the requirements for different `format` values are:
    # 
    # - `h264`
    #   - Use Annex B formatted data
    #   - Each CompressedVideo message should contain enough NAL units to decode exactly one video frame
    #   - Each message containing a key frame (IDR) must also include a SPS NAL unit
    # 
    # - `h265` (HEVC)
    #   - Use Annex B formatted data
    #   - Each CompressedVideo message should contain enough NAL units to decode exactly one video frame
    #   - Each message containing a key frame (IRAP) must also include relevant VPS/SPS/PPS NAL units
    # 
    # - `vp9`
    #   - Each CompressedVideo message should contain exactly one video frame
    # 
    # - `av1`
    #   - Use the "Low overhead bitstream format" (section 5.2)
    #   - Each CompressedVideo message should contain enough OBUs to decode exactly one video frame
    #   - Each message containing a key frame must also include a Sequence Header OBU
    # CompressedVideo
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # CompressedVideo
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # CompressedVideo
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CompressedVideo
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

    # Video format.
    # 
    # Supported values: `h264`, `h265`, `vp9`, `av1`.
    # 
    # Note: compressed video support is subject to hardware limitations and patent licensing, so not all encodings may be supported on all platforms. See more about [H.265 support](https://caniuse.com/hevc), [VP9 support](https://caniuse.com/webm), and [AV1 support](https://caniuse.com/av1).
    # CompressedVideo
    def Format(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def CompressedVideoStart(builder): builder.StartObject(4)
def Start(builder):
    return CompressedVideoStart(builder)
def CompressedVideoAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return CompressedVideoAddTimestamp(builder, timestamp)
def CompressedVideoAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return CompressedVideoAddFrameId(builder, frameId)
def CompressedVideoAddData(builder, data): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def AddData(builder, data):
    return CompressedVideoAddData(builder, data)
def CompressedVideoStartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def StartDataVector(builder, numElems):
    return CompressedVideoStartDataVector(builder, numElems)
def CompressedVideoAddFormat(builder, format): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(format), 0)
def AddFormat(builder, format):
    return CompressedVideoAddFormat(builder, format)
def CompressedVideoEnd(builder): return builder.EndObject()
def End(builder):
    return CompressedVideoEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# An array of points on a 2D image
class PointsAnnotation(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = PointsAnnotation()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsPointsAnnotation(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # PointsAnnotation
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of annotation
    # PointsAnnotation
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Type of points annotation to draw
    # PointsAnnotation
    def Type(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Points in 2D image coordinates (pixels).
    # These coordinates use the top-left corner of the top-left pixel of the image as the origin.
    # PointsAnnotation
    def Points(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Point2 import Point2
            obj = Point2()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # PointsAnnotation
    def PointsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # PointsAnnotation
    def PointsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

    # Outline color
    # PointsAnnotation
    def OutlineColor(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Per-point colors, if `type` is `POINTS`, or per-segment stroke colors, if `type` is `LINE_LIST`, `LINE_STRIP` or `LINE_LOOP`.
    # PointsAnnotation
    def OutlineColors(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # PointsAnnotation
    def OutlineColorsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # PointsAnnotation
    def OutlineColorsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        return o == 0

    # Fill color
    # PointsAnnotation
    def FillColor(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Stroke thickness in pixels
    # PointsAnnotation
    def Thickness(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

def PointsAnnotationStart(builder): builder.StartObject(7)
def Start(builder):
    return PointsAnnotationStart(builder)
def PointsAnnotationAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return PointsAnnotationAddTimestamp(builder, timestamp)
def PointsAnnotationAddType(builder, type): builder.PrependUint8Slot(1, type, 0)
def AddType(builder, type):
    return PointsAnnotationAddType(builder, type)
def PointsAnnotationAddPoints(builder, points): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(points), 0)
def AddPoints(builder, points):
    return PointsAnnotationAddPoints(builder, points)
def PointsAnnotationStartPointsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartPointsVector(builder, numElems):
    return PointsAnnotationStartPointsVector(builder, numElems)
def PointsAnnotationAddOutlineColor(builder, outlineColor): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(outlineColor), 0)
def AddOutlineColor(builder, outlineColor):
    return PointsAnnotationAddOutlineColor(builder, outlineColor)
def PointsAnnotationAddOutlineColors(builder, outlineColors): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(outlineColors), 0)
def AddOutlineColors(builder, outlineColors):
    return PointsAnnotationAddOutlineColors(builder, outlineColors)
def PointsAnnotationStartOutlineColorsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartOutlineColorsVector(builder, numElems):
    return PointsAnnotationStartOutlineColorsVector(builder, numElems)
def PointsAnnotationAddFillColor(builder, fillColor): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(fillColor), 0)
def AddFillColor(builder, fillColor):
    return PointsAnnotationAddFillColor(builder, fillColor)
def PointsAnnotationAddThickness(builder, thickness): builder.PrependFloat64Slot(6, thickness, 0.0)
def AddThickness(builder, thickness):
    return PointsAnnotationAddThickness(builder, thickness)
def PointsAnnotationEnd(builder): return builder.EndObject()
def End(builder):
    return PointsAnnotationEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Duration(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls):
        return 8

    # Duration
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Signed seconds of the span of time. Must be from -315,576,000,000 to +315,576,000,000 inclusive.
    # Duration
    def Sec(self): return self._tab.Get(flatbuffers.number_types.Int32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # if sec === 0 : -999,999,999 <= nsec <= +999,999,999
    # otherwise sign of sec must match sign of nsec or be 0 and abs(nsec) <= 999,999,999
    # Duration
    def Nsec(self): return self._tab.Get(flatbuffers.number_types.Int32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4))

def CreateDuration(builder, sec, nsec):
    builder.Prep(4, 8)
    builder.PrependInt32(nsec)
    builder.PrependInt32(sec)
    return builder.Offset()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A log message
class Log(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Log()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsLog(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Log
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of log message
    # Log
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Log level
    # Log
    def Level(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Log message
    # Log
    def Message(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Process or node name
    # Log
    def Name(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Filename
    # Log
    def File(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Line number in the file
    # Log
    def Line(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

def LogStart(builder): builder.StartObject(6)
def Start(builder):
    return LogStart(builder)
def LogAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return LogAddTimestamp(builder, timestamp)
def LogAddLevel(builder, level): builder.PrependUint8Slot(1, level, 0)
def AddLevel(builder, level):
    return LogAddLevel(builder, level)
def LogAddMessage(builder, message): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(message), 0)
def AddMessage(builder, message):
    return LogAddMessage(builder, message)
def LogAddName(builder, name): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(name), 0)
def AddName(builder, name):
    return LogAddName(builder, name)
def LogAddFile(builder, file): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(file), 0)
def AddFile(builder, file):
    return LogAddFile(builder, file)
def LogAddLine(builder, line): builder.PrependUint32Slot(5, line, 0)
def AddLine(builder, line):
    return LogAddLine(builder, line)
def LogEnd(builder): return builder.EndObject()
def End(builder):
    return LogEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A 2D grid of data
class Grid(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Grid()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsGrid(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Grid
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of grid
    # Grid
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference
    # Grid
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Origin of grid's corner relative to frame of reference; grid is positioned in the x-y plane relative to this origin
    # Grid
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Number of grid columns
    # Grid
    def ColumnCount(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Size of single grid cell along x and y axes, relative to `pose`
    # Grid
    def CellSize(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Vector2 import Vector2
            obj = Vector2()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Number of bytes between rows in `data`
    # Grid
    def RowStride(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Number of bytes between cells within a row in `data`
    # Grid
    def CellStride(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Fields in `data`. `red`, `green`, `blue`, and `alpha` are optional for customizing the grid's color.
    # Grid
    def Fields(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .PackedElementField import PackedElementField
            obj = PackedElementField()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Grid
    def FieldsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Grid
    def FieldsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        return o == 0

    # Grid cell data, interpreted using `fields`, in row-major (y-major) order
    # Grid
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # Grid
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # Grid
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Grid
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        return o == 0

def GridStart(builder): builder.StartObject(9)
def Start(builder):
    return GridStart(builder)
def GridAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return GridAddTimestamp(builder, timestamp)
def GridAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return GridAddFrameId(builder, frameId)
def GridAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return GridAddPose(builder, pose)
def GridAddColumnCount(builder, columnCount): builder.PrependUint32Slot(3, columnCount, 0)
def AddColumnCount(builder, columnCount):
    return GridAddColumnCount(builder, columnCount)
def GridAddCellSize(builder, cellSize): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(cellSize), 0)
def AddCellSize(builder, cellSize):
    return GridAddCellSize(builder, cellSize)
def GridAddRowStride(builder, rowStride): builder.PrependUint32Slot(5, rowStride, 0)
def AddRowStride(builder, rowStride):
    return GridAddRowStride(builder, rowStride)
def GridAddCellStride(builder, cellStride): builder.PrependUint32Slot(6, cellStride, 0)
def AddCellStride(builder, cellStride):
    return GridAddCellStride(builder, cellStride)
def GridAddFields(builder, fields): builder.PrependUOffsetTRelativeSlot(7, flatbuffers.number_types.UOffsetTFlags.py_type(fields), 0)
def AddFields(builder, fields):
    return GridAddFields(builder, fields)
def GridStartFieldsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartFieldsVector(builder, numElems):
    return GridStartFieldsVector(builder, numElems)
def GridAddData(builder, data): builder.PrependUOffsetTRelativeSlot(8, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def AddData(builder, data):
    return GridAddData(builder, data)
def GridStartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def StartDataVector(builder, numElems):
    return GridStartDataVector(builder, numElems)
def GridEnd(builder): return builder.EndObject()
def End(builder):
    return GridEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

# Numeric type
class NumericType(object):
    UNKNOWN = 0
    UINT8 = 1
    INT8 = 2
    UINT16 = 3
    INT16 = 4
    UINT32 = 5
    INT32 = 6
    FLOAT32 = 7
    FLOAT64 = 8



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A position and orientation for an object or reference frame in 3D space
class Pose(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Pose()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsPose(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Pose
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Point denoting position in 3D space
    # Pose
    def Position(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Vector3 import Vector3
            obj = Vector3()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Quaternion denoting orientation in 3D space
    # Pose
    def Orientation(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Quaternion import Quaternion
            obj = Quaternion()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def PoseStart(builder): builder.StartObject(2)
def Start(builder):
    return PoseStart(builder)
def PoseAddPosition(builder, position): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(position), 0)
def AddPosition(builder, position):
    return PoseAddPosition(builder, position)
def PoseAddOrientation(builder, orientation): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(orientation), 0)
def AddOrientation(builder, orientation):
    return PoseAddOrientation(builder, orientation)
def PoseEnd(builder): return builder.EndObject()
def End(builder):
    return PoseEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# Used for nesting byte vectors
class ByteVector(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ByteVector()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsByteVector(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # ByteVector
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # ByteVector
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # ByteVector
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # ByteVector
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # ByteVector
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

def ByteVectorStart(builder): builder.StartObject(1)
def Start(builder):
    return ByteVectorStart(builder)
def ByteVectorAddData(builder, data): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def AddData(builder, data):
    return ByteVectorAddData(builder, data)
def ByteVectorStartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def StartDataVector(builder, numElems):
    return ByteVectorStartDataVector(builder, numElems)
def ByteVectorEnd(builder): return builder.EndObject()
def End(builder):
    return ByteVectorEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A primitive representing a cube or rectangular prism
class CubePrimitive(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = CubePrimitive()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsCubePrimitive(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # CubePrimitive
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Position of the center of the cube and orientation of the cube
    # CubePrimitive
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Size of the cube along each axis
    # CubePrimitive
    def Size(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Vector3 import Vector3
            obj = Vector3()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Color of the cube
    # CubePrimitive
    def Color(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def CubePrimitiveStart(builder): builder.StartObject(3)
def Start(builder):
    return CubePrimitiveStart(builder)
def CubePrimitiveAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return CubePrimitiveAddPose(builder, pose)
def CubePrimitiveAddSize(builder, size): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(size), 0)
def AddSize(builder, size):
    return CubePrimitiveAddSize(builder, size)
def CubePrimitiveAddColor(builder, color): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(color), 0)
def AddColor(builder, color):
    return CubePrimitiveAddColor(builder, color)
def CubePrimitiveEnd(builder): return builder.EndObject()
def End(builder):
    return CubePrimitiveEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A point representing a position in 2D space
class Point2(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Point2()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsPoint2(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Point2
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # x coordinate position
    # Point2
    def X(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # y coordinate position
    # Point2
    def Y(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

def Point2Start(builder): builder.StartObject(2)
def Start(builder):
    return Point2Start(builder)
def Point2AddX(builder, x): builder.PrependFloat64Slot(0, x, 0.0)
def AddX(builder, x):
    return Point2AddX(builder, x)
def Point2AddY(builder, y): builder.PrependFloat64Slot(1, y, 0.0)
def AddY(builder, y):
    return Point2AddY(builder, y)
def Point2End(builder): return builder.EndObject()
def End(builder):
    return Point2End(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A transform between two reference frames in 3D space
class FrameTransform(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = FrameTransform()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsFrameTransform(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # FrameTransform
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of transform
    # FrameTransform
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Name of the parent frame
    # FrameTransform
    def ParentFrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Name of the child frame
    # FrameTransform
    def ChildFrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Translation component of the transform
    # FrameTransform
    def Translation(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Vector3 import Vector3
            obj = Vector3()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Rotation component of the transform
    # FrameTransform
    def Rotation(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Quaternion import Quaternion
            obj = Quaternion()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def FrameTransformStart(builder): builder.StartObject(5)
def Start(builder):
    return FrameTransformStart(builder)
def FrameTransformAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return FrameTransformAddTimestamp(builder, timestamp)
def FrameTransformAddParentFrameId(builder, parentFrameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(parentFrameId), 0)
def AddParentFrameId(builder, parentFrameId):
    return FrameTransformAddParentFrameId(builder, parentFrameId)
def FrameTransformAddChildFrameId(builder, childFrameId): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(childFrameId), 0)
def AddChildFrameId(builder, childFrameId):
    return FrameTransformAddChildFrameId(builder, childFrameId)
def FrameTransformAddTranslation(builder, translation): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(translation), 0)
def AddTranslation(builder, translation):
    return FrameTransformAddTranslation(builder, translation)
def FrameTransformAddRotation(builder, rotation): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(rotation), 0)
def AddRotation(builder, rotation):
    return FrameTransformAddRotation(builder, rotation)
def FrameTransformEnd(builder): return builder.EndObject()
def End(builder):
    return FrameTransformEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# An array of FrameTransform messages
class FrameTransforms(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = FrameTransforms()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsFrameTransforms(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # FrameTransforms
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Array of transforms
    # FrameTransforms
    def Transforms(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .FrameTransform import FrameTransform
            obj = FrameTransform()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # FrameTransforms
    def TransformsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # FrameTransforms
    def TransformsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

def FrameTransformsStart(builder): builder.StartObject(1)
def Start(builder):
    return FrameTransformsStart(builder)
def FrameTransformsAddTransforms(builder, transforms): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(transforms), 0)
def AddTransforms(builder, transforms):
    return FrameTransformsAddTransforms(builder, transforms)
def FrameTransformsStartTransformsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartTransformsVector(builder, numElems):
    return FrameTransformsStartTransformsVector(builder, numElems)
def FrameTransformsEnd(builder): return builder.EndObject()
def End(builder):
    return FrameTransformsEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A primitive representing a set of triangles or a surface tiled by triangles
class TriangleListPrimitive(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = TriangleListPrimitive()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsTriangleListPrimitive(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # TriangleListPrimitive
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Origin of triangles relative to reference frame
    # TriangleListPrimitive
    def Pose(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)
    # TriangleListPrimitive
    def Points(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Point3 import Point3
            obj = Point3()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # TriangleListPrimitive
    def PointsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # TriangleListPrimitive
    def PointsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        return o == 0

    # Solid color to use for the whole shape. One of `color` or `colors` must be provided.
    # TriangleListPrimitive
    def Color(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided.
    # TriangleListPrimitive
    def Colors(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # TriangleListPrimitive
    def ColorsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # TriangleListPrimitive
    def ColorsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        return o == 0

    # Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.
    # 
    # If omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided).
    # TriangleListPrimitive
    def Indices(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return 0

    # TriangleListPrimitive
    def IndicesAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint32Flags, o)
        return 0

    # TriangleListPrimitive
    def IndicesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # TriangleListPrimitive
    def IndicesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        return o == 0

def TriangleListPrimitiveStart(builder): builder.StartObject(5)
def Start(builder):
    return TriangleListPrimitiveStart(builder)
def TriangleListPrimitiveAddPose(builder, pose): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(pose), 0)
def AddPose(builder, pose):
    return TriangleListPrimitiveAddPose(builder, pose)
def TriangleListPrimitiveAddPoints(builder, points): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(points), 0)
def AddPoints(builder, points):
    return TriangleListPrimitiveAddPoints(builder, points)
def TriangleListPrimitiveStartPointsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartPointsVector(builder, numElems):
    return TriangleListPrimitiveStartPointsVector(builder, numElems)
def TriangleListPrimitiveAddColor(builder, color): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(color), 0)
def AddColor(builder, color):
    return TriangleListPrimitiveAddColor(builder, color)
def TriangleListPrimitiveAddColors(builder, colors): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(colors), 0)
def AddColors(builder, colors):
    return TriangleListPrimitiveAddColors(builder, colors)
def TriangleListPrimitiveStartColorsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartColorsVector(builder, numElems):
    return TriangleListPrimitiveStartColorsVector(builder, numElems)
def TriangleListPrimitiveAddIndices(builder, indices): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(indices), 0)
def AddIndices(builder, indices):
    return TriangleListPrimitiveAddIndices(builder, indices)
def TriangleListPrimitiveStartIndicesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartIndicesVector(builder, numElems):
    return TriangleListPrimitiveStartIndicesVector(builder, numElems)
def TriangleListPrimitiveEnd(builder): return builder.EndObject()
def End(builder):
    return TriangleListPrimitiveEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A compressed image
class CompressedImage(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = CompressedImage()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsCompressedImage(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # CompressedImage
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of image
    # CompressedImage
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference for the image. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image.
    # CompressedImage
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Compressed image data
    # CompressedImage
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # CompressedImage
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # CompressedImage
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CompressedImage
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

    # Image format
    # 
    # Supported values: image media types supported by Chrome, such as `webp`, `jpeg`, `png`
    # CompressedImage
    def Format(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def CompressedImageStart(builder): builder.StartObject(4)
def Start(builder):
    return CompressedImageStart(builder)
def CompressedImageAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return CompressedImageAddTimestamp(builder, timestamp)
def CompressedImageAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return CompressedImageAddFrameId(builder, frameId)
def CompressedImageAddData(builder, data): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def AddData(builder, data):
    return CompressedImageAddData(builder, data)
def CompressedImageStartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def StartDataVector(builder, numElems):
    return CompressedImageStartDataVector(builder, numElems)
def CompressedImageAddFormat(builder, format): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(format), 0)
def AddFormat(builder, format):
    return CompressedImageAddFormat(builder, format)
def CompressedImageEnd(builder): return builder.EndObject()
def End(builder):
    return CompressedImageEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A field present within each element in a byte array of packed elements.
class PackedElementField(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = PackedElementField()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsPackedElementField(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # PackedElementField
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Name of the field
    # PackedElementField
    def Name(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Byte offset from start of data buffer
    # PackedElementField
    def Offset(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Type of data in the field. Integers are stored using little-endian byte order.
    # PackedElementField
    def Type(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

def PackedElementFieldStart(builder): builder.StartObject(3)
def Start(builder):
    return PackedElementFieldStart(builder)
def PackedElementFieldAddName(builder, name): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(name), 0)
def AddName(builder, name):
    return PackedElementFieldAddName(builder, name)
def PackedElementFieldAddOffset(builder, offset): builder.PrependUint32Slot(1, offset, 0)
def AddOffset(builder, offset):
    return PackedElementFieldAddOffset(builder, offset)
def PackedElementFieldAddType(builder, type): builder.PrependUint8Slot(2, type, 0)
def AddType(builder, type):
    return PackedElementFieldAddType(builder, type)
def PackedElementFieldEnd(builder): return builder.EndObject()
def End(builder):
    return PackedElementFieldEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

# Type of points annotation
class PointsAnnotationType(object):
    UNKNOWN = 0
    # Individual points: 0, 1, 2, ...
    POINTS = 1
    # Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0
    LINE_LOOP = 2
    # Connected line segments: 0-1, 1-2, ..., (n-1)-n
    LINE_STRIP = 3
    # Individual line segments: 0-1, 2-3, 4-5, ...
    LINE_LIST = 4



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A raw image
class RawImage(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = RawImage()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsRawImage(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # RawImage
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of image
    # RawImage
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference for the image. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image.
    # RawImage
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Image width
    # RawImage
    def Width(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Image height
    # RawImage
    def Height(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Encoding of the raw image data
    # 
    # Supported values: `8UC1`, `8UC3`, `16UC1` (little endian), `32FC1` (little endian), `bayer_bggr8`, `bayer_gbrg8`, `bayer_grbg8`, `bayer_rggb8`, `bgr8`, `bgra8`, `mono8`, `mono16`, `rgb8`, `rgba8`, `uyvy` or `yuv422`, `yuyv` or `yuv422_yuy2`
    # RawImage
    def Encoding(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Byte length of a single row
    # RawImage
    def Step(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Raw image data
    # RawImage
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # RawImage
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # RawImage
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # RawImage
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        return o == 0

def RawImageStart(builder): builder.StartObject(7)
def Start(builder):
    return RawImageStart(builder)
def RawImageAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return RawImageAddTimestamp(builder, timestamp)
def RawImageAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return RawImageAddFrameId(builder, frameId)
def RawImageAddWidth(builder, width): builder.PrependUint32Slot(2, width, 0)
def AddWidth(builder, width):
    return RawImageAddWidth(builder, width)
def RawImageAddHeight(builder, height): builder.PrependUint32Slot(3, height, 0)
def AddHeight(builder, height):
    return RawImageAddHeight(builder, height)
def RawImageAddEncoding(builder, encoding): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(encoding), 0)
def AddEncoding(builder, encoding):
    return RawImageAddEncoding(builder, encoding)
def RawImageAddStep(builder, step): builder.PrependUint32Slot(5, step, 0)
def AddStep(builder, step):
    return RawImageAddStep(builder, step)
def RawImageAddData(builder, data): builder.PrependUOffsetTRelativeSlot(6, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def AddData(builder, data):
    return RawImageAddData(builder, data)
def RawImageStartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def StartDataVector(builder, numElems):
    return RawImageStartDataVector(builder, numElems)
def RawImageEnd(builder): return builder.EndObject()
def End(builder):
    return RawImageEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A [quaternion](https://eater.net/quaternions) representing a rotation in 3D space
class Quaternion(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Quaternion()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsQuaternion(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Quaternion
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # x value
    # Quaternion
    def X(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # y value
    # Quaternion
    def Y(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # z value
    # Quaternion
    def Z(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # w value
    # Quaternion
    def W(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 1.0

def QuaternionStart(builder): builder.StartObject(4)
def Start(builder):
    return QuaternionStart(builder)
def QuaternionAddX(builder, x): builder.PrependFloat64Slot(0, x, 0.0)
def AddX(builder, x):
    return QuaternionAddX(builder, x)
def QuaternionAddY(builder, y): builder.PrependFloat64Slot(1, y, 0.0)
def AddY(builder, y):
    return QuaternionAddY(builder, y)
def QuaternionAddZ(builder, z): builder.PrependFloat64Slot(2, z, 0.0)
def AddZ(builder, z):
    return QuaternionAddZ(builder, z)
def QuaternionAddW(builder, w): builder.PrependFloat64Slot(3, w, 1.0)
def AddW(builder, w):
    return QuaternionAddW(builder, w)
def QuaternionEnd(builder): return builder.EndObject()
def End(builder):
    return QuaternionEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# An array of timestamped poses for an object or reference frame in 3D space
class PosesInFrame(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = PosesInFrame()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsPosesInFrame(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # PosesInFrame
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of pose
    # PosesInFrame
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Frame of reference for pose position and orientation
    # PosesInFrame
    def FrameId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Poses in 3D space
    # PosesInFrame
    def Poses(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Pose import Pose
            obj = Pose()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # PosesInFrame
    def PosesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # PosesInFrame
    def PosesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

def PosesInFrameStart(builder): builder.StartObject(3)
def Start(builder):
    return PosesInFrameStart(builder)
def PosesInFrameAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return PosesInFrameAddTimestamp(builder, timestamp)
def PosesInFrameAddFrameId(builder, frameId): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(frameId), 0)
def AddFrameId(builder, frameId):
    return PosesInFrameAddFrameId(builder, frameId)
def PosesInFrameAddPoses(builder, poses): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(poses), 0)
def AddPoses(builder, poses):
    return PosesInFrameAddPoses(builder, poses)
def PosesInFrameStartPosesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartPosesVector(builder, numElems):
    return PosesInFrameStartPosesVector(builder, numElems)
def PosesInFrameEnd(builder): return builder.EndObject()
def End(builder):
    return PosesInFrameEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# An update to the entities displayed in a 3D scene
class SceneUpdate(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = SceneUpdate()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsSceneUpdate(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # SceneUpdate
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Scene entities to delete
    # SceneUpdate
    def Deletions(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .SceneEntityDeletion import SceneEntityDeletion
            obj = SceneEntityDeletion()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneUpdate
    def DeletionsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneUpdate
    def DeletionsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

    # Scene entities to add or replace
    # SceneUpdate
    def Entities(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .SceneEntity import SceneEntity
            obj = SceneEntity()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # SceneUpdate
    def EntitiesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SceneUpdate
    def EntitiesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        return o == 0

def SceneUpdateStart(builder): builder.StartObject(2)
def Start(builder):
    return SceneUpdateStart(builder)
def SceneUpdateAddDeletions(builder, deletions): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(deletions), 0)
def AddDeletions(builder, deletions):
    return SceneUpdateAddDeletions(builder, deletions)
def SceneUpdateStartDeletionsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartDeletionsVector(builder, numElems):
    return SceneUpdateStartDeletionsVector(builder, numElems)
def SceneUpdateAddEntities(builder, entities): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(entities), 0)
def AddEntities(builder, entities):
    return SceneUpdateAddEntities(builder, entities)
def SceneUpdateStartEntitiesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartEntitiesVector(builder, numElems):
    return SceneUpdateStartEntitiesVector(builder, numElems)
def SceneUpdateEnd(builder): return builder.EndObject()
def End(builder):
    return SceneUpdateEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A text label on a 2D image
class TextAnnotation(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = TextAnnotation()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsTextAnnotation(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # TextAnnotation
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Timestamp of annotation
    # TextAnnotation
    def Timestamp(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Time import Time
            obj = Time()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Bottom-left origin of the text label in 2D image coordinates (pixels).
    # The coordinate uses the top-left corner of the top-left pixel of the image as the origin.
    # TextAnnotation
    def Position(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Point2 import Point2
            obj = Point2()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Text to display
    # TextAnnotation
    def Text(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Font size in pixels
    # TextAnnotation
    def FontSize(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 12.0

    # Text color
    # TextAnnotation
    def TextColor(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Background fill color
    # TextAnnotation
    def BackgroundColor(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Color import Color
            obj = Color()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def TextAnnotationStart(builder): builder.StartObject(6)
def Start(builder):
    return TextAnnotationStart(builder)
def TextAnnotationAddTimestamp(builder, timestamp): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(timestamp), 0)
def AddTimestamp(builder, timestamp):
    return TextAnnotationAddTimestamp(builder, timestamp)
def TextAnnotationAddPosition(builder, position): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(position), 0)
def AddPosition(builder, position):
    return TextAnnotationAddPosition(builder, position)
def TextAnnotationAddText(builder, text): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(text), 0)
def AddText(builder, text):
    return TextAnnotationAddText(builder, text)
def TextAnnotationAddFontSize(builder, fontSize): builder.PrependFloat64Slot(3, fontSize, 12.0)
def AddFontSize(builder, fontSize):
    return TextAnnotationAddFontSize(builder, fontSize)
def TextAnnotationAddTextColor(builder, textColor): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(textColor), 0)
def AddTextColor(builder, textColor):
    return TextAnnotationAddTextColor(builder, textColor)
def TextAnnotationAddBackgroundColor(builder, backgroundColor): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(backgroundColor), 0)
def AddBackgroundColor(builder, backgroundColor):
    return TextAnnotationAddBackgroundColor(builder, backgroundColor)
def TextAnnotationEnd(builder): return builder.EndObject()
def End(builder):
    return TextAnnotationEnd(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: foxglove

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Time(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls):
        return 8

    # Time
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Represents seconds of UTC time since Unix epoch 1970-01-01T00:00:00Z
    # Time
    def Sec(self): return self._tab.Get(flatbuffers.number_types.Uint32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # Nano-second fractions from 0 to 999,999,999 inclusive
    # Time
    def Nsec(self): return self._tab.Get(flatbuffers.number_types.Uint32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4))

def CreateTime(builder, sec, nsec):
    builder.Prep(4, 8)
    builder.PrependUint32(nsec)
    builder.PrependUint32(sec)
    return builder.Offset()
